/***************************************************************************/
/* 		This code is part of Nscache - viewer of Netscape(tm)	   */
/*		browsers disk cache					   */
/*		Copyright (c) 1999,2000 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		modified 2005,2006,2007 by Harald Foerster		   */
/*		(harald_foerster@users.sourceforge.net)			   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include <ctype.h>
#include <string.h>
#include <unistd.h>

#include "gui.h"
#include "file.h"
#include "gprop.h"
#include "nls.h"
#include "re.h"

#ifdef HAVE_FNMATCH
#include <fnmatch.h>
#else
#include "fnmatch.h"
#endif

/*
#include "gui-gtk.h"
#include "nscache.h"
*/


enum
{
	kColumnURL,
	kColumnFound,
	kColumnFile,
#ifndef CONFIG_GTKCLIST_GTKCTREE
	kColumnPixbuf,
	kColumnRowData,
#endif
	kNumberOfColumns
};


#ifdef CONFIG_GTKCLIST_GTKCTREE
#define SearchListNew(list) \
	GuiListInit((list), kNumberOfColumns)
#else
#define SearchListNew(list) \
{ \
	(list)->slist = \
		gtk_list_store_new (kNumberOfColumns, G_TYPE_STRING, \
					G_TYPE_STRING, G_TYPE_STRING, \
					GDK_TYPE_PIXBUF, G_TYPE_POINTER); \
	GuiListInit((list), kColumnRowData); \
}
#endif

#ifdef CONFIG_GTKCLIST_GTKCTREE
#define SearchListAppend(list, str, icon, data) \
{ \
	gint row; \
	gtk_clist_freeze((list)->clist); \
	row = gtk_clist_append((list)->clist, (str)); \
	gtk_clist_set_pixtext((list)->clist, row, 0, (str)[0], \
			3, (icon)->pixmap, (icon)->mask); \
	gtk_clist_set_row_data((list)->clist, row, (data)); \
	gtk_clist_thaw((list)->clist); \
}
#else
#define SearchListAppend(list, str, icon, data) \
{ \
	GtkTreeIter iter; \
	gtk_list_store_append((list)->slist, &iter); \
	gtk_list_store_set((list)->slist, &iter, \
		kColumnURL, (str)[0], kColumnFound, (str)[1], \
		kColumnFile, (str)[2], kColumnPixbuf, (icon)->pixbuf, \
		kColumnRowData, (data), -1); \
}
#endif


typedef char* (*StrStrFunc) (const char *, const char *);

typedef struct
{
	GtkWidget	*window;
	GtkEntry	*pattern_w;
	GtkEntry	*mime_type_w;
	GtkWidget	*case_w;
	GtkWidget	*full_match_w;

#ifdef HAVE_REGEX
	GtkWidget	*re_w;
#endif

	GtkWidget	*progress_bar;
	GtkWidget	*do_search;
	GtkWidget	*do_stop;
	GtkWidget	*do_close;

	GtkWidget	*search_all;
	GtkWidget	*search_selected;
	GtkWidget	*search_results;

	GSList		*docs_slist;
	guint		docs_processed;
	guint		docs_total;
	guint		idle_id;

	gboolean	full_match;
	StrStrFunc	strstrfunc;
	char		*pattern_str;
	size_t		pattern_len;
	char		*mime_type;

#ifdef HAVE_REGEX
	re_entry	*reg_exp;
#endif

	ListSelection	search_selection;

} search_info_t;


GSList *search_dialog_list = NULL;

static char search_dialog_title[] = gettext_nop("MozCache: Find");

static void search_append_cache_doc(nscache_record_t *nscrec, search_info_t *info)
{
	if (nscrec)
	{
		if ( info->mime_type == NULL ||
			info->mime_type == nscrec->content_type ||
				(nscrec->content_type == NULL ?
					strcmp(info->mime_type, "*") :
					fnmatch(info->mime_type,
						nscrec->content_type, 0)) == 0 )
					
		{
			info->docs_slist = g_slist_append(info->docs_slist, nscrec);
			info->docs_total++;
		}
	}
}

static void search_get_cache_docs_from_tree(GuiTreeStore *tree,
					nscache_record_t* nscrec, search_info_t *info)
{
	search_append_cache_doc(nscrec, info);
}

static void search_get_cache_docs(search_info_t *info)
{
	if (mainlist->name == NULL)
	{
		return;
	}

	if (GTK_TOGGLE_BUTTON(info->search_all)->active)
	{
		guitl_tree_recursive(&(notebook_list[kTreeList].store.tree), FALSE,
			(GuiTreeStoreRecursiveFunc) search_get_cache_docs_from_tree, info);
	}

	else if (GTK_TOGGLE_BUTTON(info->search_selected)->active)
	{
		if (mainlist->nscrec)
		{
			search_append_cache_doc(mainlist->nscrec, info);
		}

		else
		{
			guitl_tree_recursive(&(notebook_list[kTreeList].store.tree), TRUE,
				(GuiTreeStoreRecursiveFunc) search_get_cache_docs_from_tree, info);
		}
	}

	else if (GTK_TOGGLE_BUTTON(info->search_results)->active)
	{
		guitl_list_foreach_row_data(&(info->search_selection.store.list),
						(GFunc) search_append_cache_doc, info);
	}
}

static void search_dlg_free_runtime_data(search_info_t *info)
{
	if (info->idle_id)
	{
		GuiIdleRemove(info->idle_id);
		info->idle_id = 0;
	}

	g_free(info->pattern_str);
	info->pattern_str = NULL;

	g_free(info->mime_type);
	info->mime_type = NULL;

#ifdef HAVE_REGEX

	if (info->reg_exp)
	{
		re_free(info->reg_exp);
		info->reg_exp = NULL;
	}
#endif

	g_slist_free(info->docs_slist);
	info->docs_slist = NULL;
}

static void search_dlg_destroy(GtkWidget *w, search_info_t *info)
{
	list_selection_chain =
		g_slist_remove(list_selection_chain, &(info->search_selection));

	search_dialog_list = g_slist_remove(search_dialog_list, info);

	search_dlg_free_runtime_data(info);

#ifndef CONFIG_GTKCLIST_GTKCTREE
	g_free(info->search_selection.name);
#endif

	g_free(info);
}

static char *search_dlg_search_file(search_info_t *info, char *beg, char *end)
{
	int end_offs = info->pattern_len;

	while (end > beg)
	{
		char 	*start;
		size_t	len;

		if ( ! isprint(*beg))
		{
			beg++;
			continue;
		}

		if ('\r' == '\n')
			len = strcspn(beg, "\x0D\x0A");
		else
			len = strcspn(beg, "\r\n");

		beg[len] = '\0';

		start = beg;

		for (;;)
		{

#ifdef HAVE_REGEX
			if (info->strstrfunc == NULL)
			{
				int	beg_offs;
				size_t	slen;

				slen	 = &beg[len] - start;
				beg_offs = RegexSearch(info->reg_exp, start, slen, &end_offs);

				if (beg_offs < 0)
				{
					break;
				}

				if (end_offs < 0)
				{
					/* no offset information */

					if (len > 255)
					{
						beg[255] = '\0';
					}

					return beg;
				}

				end_offs -= beg_offs;
				start	 += beg_offs;
			}
			else
#endif /* HAVE_REGEX */
			{
				start = info->strstrfunc(start, info->pattern_str);

				if (start == NULL)
				{
					break;
				}
			}

			if ( ( ! info->full_match) ||
					(( start == beg || ! isalnum(start[-1]) ) &&
								! isalnum(start[end_offs])) )
			{
				if (len > 255)
				{
					beg[255] = '\0';
				}

				return beg;
			}

			start++;

		} /* for (;;) */

		beg += len + 1;

	} /* while (end > beg) */

	return NULL;
}

static void search_disable_results_radiobutton(search_info_t *info)
{
	list_selection_chain =
		g_slist_remove(list_selection_chain, &(info->search_selection));

#ifndef CONFIG_GTKCLIST_GTKCTREE
	GuiListSetSelectionMode(&(info->search_selection.store.list),
							GTK_SELECTION_NONE);
	g_free(info->search_selection.name);
#endif

	info->search_selection.name   = NULL;
	info->search_selection.nscrec = NULL;

	if (GTK_TOGGLE_BUTTON(info->search_results)->active)
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(info->search_all), TRUE);
	}

	gtk_widget_set_sensitive(info->search_results, FALSE);
}

static void search_disable_all_entries(GuiListStore *list, gpointer userdata)
{
	GuiListSetRowData(list, userdata);

#ifdef CONFIG_GTKCLIST_GTKCTREE
	gtk_clist_set_selectable(list->clist, list->row, FALSE);
#endif
}

void search_disable_entries(gpointer data, gpointer user)
{
	/* if there are search dialogs containing results */

	search_info_t	*info;
	GuiListStore	*list;

	info = data;
	list = &(info->search_selection.store.list);

	guitl_list_foreach(list,
		(GuiListStoreForeachFunc) search_disable_all_entries, NULL);

	info->docs_total = -1;

	search_disable_results_radiobutton(info);
}

gint search_update_window_title(gpointer data, gpointer user)
{
	gint	found;
	gchar	*str;
	gchar	pom[256];

	search_info_t *info = data;

	if (info->docs_total == -1)
	{
		return 0; /* no search started */
	}

	found = GuiListGetRows(&(info->search_selection.store.list));

	if (found)
	{
		gtk_widget_set_sensitive(info->search_results, TRUE);
		g_snprintf(pom, sizeof(pom),
				gettext("MozCache: Find (found %d)"), found);
		str = utf8_from_str(pom);
		gtk_window_set_title(GTK_WINDOW(info->window), str == NULL ? "???" : str);
	}

	else
	{
		search_disable_results_radiobutton(info);
		str = guitl_gettext_utf8_from_str(gettext_nop("MozCache: Find (not found)"));
		gtk_window_set_title(GTK_WINDOW(info->window), str);
	}

	utf8_free(str);

	return found;
}

static void search_dlg_search_stop(GtkWidget *w, search_info_t *info)
{
	search_dlg_free_runtime_data(info);

	gtk_window_set_modal(GTK_WINDOW(info->window), FALSE);

	gtk_widget_set_sensitive(info->do_stop, FALSE);
	gtk_widget_set_sensitive(info->do_search, TRUE);
	gtk_widget_hide(info->progress_bar);

	if (search_update_window_title(info, NULL) > 0)
	{
		list_selection_chain =
			g_slist_append(list_selection_chain, &(info->search_selection));

		GuiListSetSelectionMode(&(info->search_selection.store.list),
								GTK_SELECTION_BROWSE);

		GuiListSelectFirstRow(&(info->search_selection.store.list));
	}

	gdk_beep();
}

static gint search_dlg_search_doc(search_info_t *info)
{
	int	len;
	char	*fbuf;
	nscache_record_t *nscrec;

	if (info->docs_slist == NULL)
	{
		info->idle_id = 0;
		search_dlg_search_stop(NULL, info);
		return FALSE;
	}

	nscrec = info->docs_slist->data;
	info->docs_slist = g_slist_remove(info->docs_slist, nscrec);
	info->docs_processed++;

	GuiProgressBarSetValue(GTK_PROGRESS_BAR(info->progress_bar),
		((gdouble) info->docs_processed) / ((gdouble) info->docs_total));

	fbuf = file_read(nscrec, &len);

	if (fbuf == NULL)
	{
		perror(nscrec->filename);
	}

	else
	{
		char *matchln = search_dlg_search_file(info, fbuf, &fbuf[len]);

		if (matchln)
		{
			const PixbufIcon *icon;
			char  *pp[3];

			pp[kColumnURL]   = nscrec->urlstr;
			pp[kColumnFile]	 = nscrec->filename;
			pp[kColumnFound] = utf8_from_str(matchln);

			if (pp[kColumnFound] == NULL)
			{
				pp[kColumnFound] = g_strdup("???");
			}

			icon = guitl_get_icon_mime(nscrec->content_type);

			SearchListAppend(&(info->search_selection.store.list),
									pp, icon, nscrec);
			utf8_free(pp[kColumnFound]);
		}

		free(fbuf);
	}

	return TRUE;
}

static void search_dlg_start(GtkWidget *w, search_info_t *info)
{
	const gchar	*mime_entry, *search_entry;
	gchar		*str;
	int		case_sensitive;

#ifdef HAVE_REGEX
	gboolean	with_reg_exp;
#endif

	search_dlg_free_runtime_data(info);

	info->full_match = GTK_TOGGLE_BUTTON(info->full_match_w)->active;

	mime_entry = gtk_entry_get_text(info->mime_type_w);

	if (mime_entry == NULL || *mime_entry == '\0')
	{
		str = NULL;
	}

	else
	{
		str = utf8_to_strdup(mime_entry);

		if (str == NULL)
		{
			gdk_beep();
			return;
		}
	}

	info->mime_type = str;

	search_entry = gtk_entry_get_text(info->pattern_w);

	if (search_entry == NULL || *search_entry == '\0' ||
			(str = utf8_to_str(search_entry)) == NULL)
	{
		gdk_beep();
		return;
	}

	info->pattern_str = g_strdup(str);
	info->pattern_len = strlen(info->pattern_str);

	case_sensitive = GTK_TOGGLE_BUTTON(info->case_w)->active;

	if (case_sensitive)
	{
		info->strstrfunc = (StrStrFunc) strstr;
	}

	else
	{
		info->strstrfunc = (StrStrFunc) tl_igncase_strstr;
		g_strup(info->pattern_str); /* upper case */
	}

#ifdef HAVE_REGEX

	with_reg_exp = GTK_TOGGLE_BUTTON(info->re_w)->active;

	if (with_reg_exp)
	{
		char *error;

		info->reg_exp = re_make(info->pattern_str, !case_sensitive, &error);

		if (info->reg_exp == NULL)
		{
			static const char errstr[] =
				gettext_nop("Error compiling regular expression");

			if (error == NULL)
			{
				gui_errfmt("%s : %s",
						gettext(errstr), str);
			}

			else
			{
				gui_errfmt("%s : %s - %s",
						gettext(errstr), str, error);
				g_free(error);
			}

			utf8_free(str);
			search_dlg_free_runtime_data(info);

			return;
		}

		info->strstrfunc = (StrStrFunc) NULL;
	}

	gprop_set_bool(kPropBoolFindWithRegex, with_reg_exp);

#endif /* HAVE_REGEX */

	gprop_set_str(kPropStrFindPattern, str);
	gprop_set_str(kPropStrFindMimeType, info->mime_type);
	gprop_set_bool(kPropBoolFindCaseSensitive, case_sensitive);
	gprop_set_bool(kPropBoolFindFullMatch, info->full_match);

	utf8_free(str);

	str = guitl_gettext_utf8_from_str(search_dialog_title);
	gtk_window_set_title(GTK_WINDOW(info->window), str);
	utf8_free(str);

	gtk_window_set_modal(GTK_WINDOW(info->window), TRUE);

	info->docs_total = info->docs_processed = 0;
	search_get_cache_docs(info);

	GuiListClear(&(info->search_selection.store.list));

#ifndef CONFIG_GTKCLIST_GTKCTREE
	g_free(info->search_selection.name);
#endif
	info->search_selection.name = NULL;
	info->search_selection.nscrec = NULL;

	if (info->docs_total == 0)
	{
		search_dlg_search_stop(NULL, info);
	}

	else
	{
		list_selection_chain =
			g_slist_remove(list_selection_chain, &(info->search_selection));

		gtk_widget_set_sensitive(info->do_stop, TRUE);
		gtk_widget_set_sensitive(info->do_search, FALSE);

		GuiProgressBarSetValue(GTK_PROGRESS_BAR(info->progress_bar), 0.0);
		gtk_widget_show(info->progress_bar);

		info->idle_id = GuiIdleAddPriority(GUI_PRIORITY_REDRAW,
							search_dlg_search_doc, info);
	}
}

#ifdef CONFIG_GTKCLIST_GTKCTREE
static void search_dlg_result_list_resize_column(GtkWidget *w, gint colnr,
							gint width, gpointer data)
{
	/* data == kPropIntFindColWidthUrl */

	int prop = (int) data;

	gprop_set_int(prop + colnr, width);
}
#endif

static GtkWidget *search_tab_add_check_button(GtkTable *table, gchar row,
							gchar *str, gint prop)
{
	gint		sensitive;
	GtkWidget	*check_button;

	str = guitl_gettext_utf8_from_str(str);
	check_button = gtk_check_button_new_with_label(str);
	utf8_free(str);

	gtk_table_attach(table, check_button, 0, 1, row, row + 1,
						GTK_FILL, GTK_FILL, 2, 2);
	if (gprop_get_bool(prop, &sensitive))
	{
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), sensitive);
	}

	return check_button;
}

static GtkWidget *search_tab_add_radio_button(GtkTable *table, gchar row,
						gchar *str, GtkWidget *radio_button)
{
	GSList *button_group;

	str = guitl_gettext_utf8_from_str(str);

	if ((button_group = (GSList *) radio_button) != NULL)
	{
		button_group = GuiRadioButtonGetGroup(GTK_RADIO_BUTTON(radio_button));
	}

	radio_button = gtk_radio_button_new_with_label(button_group, str);
	utf8_free(str);

	gtk_table_attach(table, radio_button, 1, 2, row, row + 1,
						GTK_FILL, GTK_FILL, 16, 2);
	return radio_button;
}

static void search_build_column(GuiListStore *list, gint colnr, gchar *title)
{
	gint prop, width;

	width = 100;

	if (colnr == kColumnURL)
	{
		GuiListColumnPixtextNew(list, kColumnURL, kColumnPixbuf);
		prop = kPropIntFindColWidthUrl;
	}

	else
	{
		GuiListColumnTextNew(list, colnr, GTK_JUSTIFY_LEFT);

		prop = (colnr == kColumnFound ?
					kPropIntFindColWidthFound :
						kPropIntFindColWidthFile);
	}

	gprop_get_int(prop, &width);

#ifndef CONFIG_GTKCLIST_GTKCTREE
	GuiListSignalConnectResizeColumn(list, colnr,
				gui_list_resize_column, (gpointer) prop);
#endif

	title = guitl_gettext_utf8_from_str(title);
	GuiListSetColumnTitle(list, colnr, title);
	utf8_free(title);

	GuiListSetColumnAutoResize(list, colnr, FALSE);
	GuiListSetColumnResizable(list, colnr, TRUE);
	GuiListSetColumnWidth(list, colnr, width);
}

void search_dlg_open(void)
{
	gchar		*p, *str;
	GtkBox		*vbox, *hbox, *tbox;
	GtkCombo	*combo;
	GtkTable	*table;
	GtkWidget	*bbox, *frame, *swin, *widget;
	search_info_t	*info;

	info = g_malloc(sizeof(search_info_t));
	search_dialog_list = g_slist_append(search_dialog_list, info);
	memset(info, '\0', sizeof(search_info_t));

	info->docs_total = -1; /* no search started */

	info->window = gui_window_new(search_dialog_title, kPropIntFindWinWidth);
	gtk_container_set_border_width(GTK_CONTAINER(info->window), 5);
	GuiSignalConnect(info->window, "destroy", search_dlg_destroy, info);

	widget = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(info->window), widget);
	vbox = GTK_BOX(widget);

	widget = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(vbox, widget, FALSE, FALSE, 2);
	hbox = GTK_BOX(widget);

	widget = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(widget), 2);
	frame = guitl_frame_new_add_child(NULL, widget);
	gtk_box_pack_start(hbox, frame, TRUE, TRUE, 0);

	tbox = GTK_BOX(widget);
	widget = gtk_table_new(2, 2, FALSE);
	gtk_box_pack_start(tbox, widget, TRUE, TRUE, 0);
	table = GTK_TABLE(widget);

	info->pattern_w =
		guitl_tab_add_entry(table, gettext_nop("Pattern: "), 0, 0);

	if (gprop_get_str(kPropStrFindPattern, &p) && p != NULL)
	{
		str = utf8_from_str(p);

		if (str)
		{
			gtk_entry_set_text(info->pattern_w, str);
			utf8_free(str);
		}
	}

	GuiSignalConnect(info->pattern_w, "activate", search_dlg_start, info);

	combo = guitl_tab_add_combo(table, gettext_nop("MIME type: "),
						mime_types_available, 0, 1);
	info->mime_type_w = GTK_ENTRY(combo->entry);

	if (gprop_get_str(kPropStrFindMimeType, &p) && p != NULL &&
			g_list_find_custom(mime_types_available, p,
							(GCompareFunc) strcmp))
	{
		str = utf8_from_str(p);

		if (str)
		{
			gtk_entry_set_text(info->mime_type_w, str);
			utf8_free(str);
		}
	}

	widget = gtk_table_new(2, 3, FALSE);
	gtk_box_pack_start(tbox, widget, TRUE, TRUE, 0);
	table = GTK_TABLE(widget);

	info->case_w = search_tab_add_check_button(table, 0,
		gettext_nop("Case sensitive"), kPropBoolFindCaseSensitive);

	info->full_match_w = search_tab_add_check_button(table, 1,
		gettext_nop("Match whole word"), kPropBoolFindFullMatch);

#ifdef HAVE_REGEX

	info->re_w = search_tab_add_check_button(table, 2,
		gettext_nop("Regular pattern"), kPropBoolFindWithRegex);
#endif

	info->search_all = search_tab_add_radio_button(table, 0,
			gettext_nop("All entries"), NULL);

	info->search_selected = search_tab_add_radio_button(table, 1,
			gettext_nop("Selected folder/entry"), info->search_all);

	info->search_results = search_tab_add_radio_button(table, 2,
			gettext_nop("Results"), info->search_selected);

	gtk_widget_set_sensitive(info->search_results, FALSE);

	bbox = gtk_vbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_EDGE);
	gtk_box_pack_start(hbox, bbox, FALSE, FALSE, 0);

	info->do_search = guitl_button_box_add_icon_button(bbox, kIconButtonSearch);
	GuiSignalConnect(info->do_search, "clicked", search_dlg_start, info);

	info->do_stop = guitl_button_box_add_icon_button(bbox, kIconButtonStop);
	GuiSignalConnect(info->do_stop, "clicked", search_dlg_search_stop, info);
	gtk_widget_set_sensitive(info->do_stop, FALSE);

	info->do_close = guitl_button_box_add_icon_button(bbox, kIconButtonClose);
	GuiSignalConnectSwapped(info->do_close, "clicked", gtk_widget_destroy, info->window);

	widget = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(widget), 2);
	swin = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add(GTK_CONTAINER(widget), swin);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (swin),
				GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);

	frame = guitl_frame_new_add_child(gettext_nop("Results"), widget);
	gtk_container_add(GTK_CONTAINER(vbox), frame);

	info->search_selection.type = kSortedList;

	SearchListNew(&(info->search_selection.store.list));	
	GuiListContainerAdd(&(info->search_selection.store.list), swin);

	GuiObjectSetUserData(info->window, &(info->search_selection));

	search_build_column(&(info->search_selection.store.list), kColumnURL,
									gettext_nop("URL"));
	search_build_column(&(info->search_selection.store.list), kColumnFound,
									gettext_nop("Found"));
	search_build_column(&(info->search_selection.store.list), kColumnFile,
									gettext_nop("Local file"));

	GuiListColumnTitlesShow(&(info->search_selection.store.list));
	GuiListColumnTitlesPassive(&(info->search_selection.store.list));

	GuiListSignalConnect(&(info->search_selection.store.list),
		"button-press-event", gui_mouse_button_pressed, gui_popup_menu);

#if GTK_MAJOR_VERSION == 1

	GuiSignalConnect(info->window, "key-press-event",
				gui_sf10_show_popup_menu, gui_popup_menu);
#else
	GuiSignalConnect(info->window, "popup-menu",
				gui_sf10_show_popup_menu, gui_popup_menu);
#endif

#ifdef CONFIG_GTKCLIST_GTKCTREE
	GuiListSignalConnectResizeColumn(&(info->search_selection.store.list), 
					search_dlg_result_list_resize_column,
						(gpointer) kPropIntFindColWidthUrl);
#endif

	GuiListSignalConnectSelectedRow(&(info->search_selection.store.list),
				gui_list_row_selected, &(info->search_selection));
	GuiListSignalConnectUnselectedRow(&(info->search_selection.store.list),
				gui_list_row_unselected, &(info->search_selection));

	gtk_widget_show_all(info->window);

	info->progress_bar = gtk_progress_bar_new();
	gtk_box_pack_start(vbox, info->progress_bar, FALSE, FALSE, 0);
}

/* EOF */
