/***************************************************************************/
/* 		This code is part of Nscache - viewer of Netscape(tm)	   */
/*		browsers disk cache					   */
/*		Copyright (c) 1999,2000 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		modified 2005,2006,2007 by Harald Foerster		   */
/*		(harald_foerster@users.sourceforge.net)			   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include <errno.h>
#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#else
#include "getopt.h"
#endif

#include "apassign.h"
#include "nls.h"
#include "gaccel.h"
#include "gprop.h"
#include "nscache.h"

enum
{
	kPrintVersion	= 1 << 0,
	kPrintHelp	= 1 << 1
};

enum
{
	NSC_FIRST,
	NSC_TYPE = NSC_FIRST,
	NSC_INDEXDB,
	NSC_VIEWER,
	NSC_MENUACCEL,
	NSC_PROP,
	NSC_LAST = NSC_PROP
};

static const char *cfgrc_name[] =
{
	"CacheType:",
	"IndexDB:",
	"Viewer:",
	"MenuAccel:",
	"Property:"
};

static const char* nscache_cache_type_str[] =
{
#ifdef CONFIG_BERKELEY_DB
	"ns",
	"ns6",
#endif
	"moz",

#ifdef HAVE_MSIE
	"ie",
#endif

	"unk",
	NULL
};

static const char illegal_cache_type_str[] = gettext_nop("Illegal cache type");

static const char* nscache_sort_type_str[] =
{
	"url",
	"file",
	"size",
	"access",
	"mime",
	NULL
};

static int get_str_index(const char **tab, const char *pattern)
{
	int		idx;
	const char	*str;

	idx = 0;

	while ((str = *(tab++)) != NULL)
	{
		if (strcmp(str, pattern) == 0)
		{
			return idx;
		}

		idx++;
	}

	return -1; /* no match */
}

static void save_apassign_data(apassign *ad, FILE *fd)
{
	static const char escape_chars[] = "\\\"";

	int	len;
	char	*mt, *fv, *uv;
	char	sbuf[3 * 1024];

	mt  = sbuf;
	len = tl_str_escape_chars(mt, sizeof(sbuf),
						ad->mimetype,  escape_chars);
	fv  = &mt[len + 1];
	len = tl_str_escape_chars(fv, &sbuf[sizeof(sbuf)] - fv,
						ad->args_file, escape_chars);
	uv  = &fv[len + 1];
	tl_str_escape_chars(uv, &sbuf[sizeof(sbuf)] - uv,
						ad->args_url, escape_chars);

	fprintf(fd, "%s \"%s\" \"%s\" \"%s\"\n", cfgrc_name[NSC_VIEWER], mt, fv, uv);
}

static FILE *open_rc(const char *modus)
{
	FILE		*fd;
	char		*rc;
	const char	*home;

	home = g_get_home_dir();

	if (home == NULL)
	{
		return (FILE *) home;
	}

	rc = g_strconcat(home, "/.mozcacherc", NULL);
	fd = fopen(rc, modus);

	g_free(rc);

	return fd;
}

void save_rc(void)
{
	FILE *fd = open_rc("w+");

	if (fd == NULL)
	{
		gui_err(gettext("Unable to open ~/.mozcacherc file"));
		return;
	}

	fprintf(fd, "# %s\n# %s\n\n",
			gettext("This .mozcacherc file was generated by MozCache"),
					gettext("You may edit it if you're careful!"));
	if (nscache_db.name)
	{
		const char *path;

		if (nscache_db.type < NSCACHE_RT_UNKNOWN)
		{
			fprintf(fd, "%s %s\n", cfgrc_name[NSC_TYPE],
					nscache_cache_type_str[nscache_db.type]);
		}

		path = nscache_db.path.str;

		fprintf(fd, "%s %s%s\n\n", cfgrc_name[NSC_INDEXDB],
				(path == NULL ? "" : path), nscache_db.name);
	}

	g_slist_foreach(apassign_data, (GFunc) save_apassign_data, fd);

	gaccel_save_keys(fd);
	gprop_save(fd);

	fclose(fd);
}

static char *get_apassign_str(char *line)
{
	char *str = tl_get_1qstr(line);

	if (str == NULL || *str == '\0')
	{
		return NULL;
	}

	return g_strdup(str);
}

static void fprn_stderr(const char *format, const char *mesg, const char *detail)
{
	if (format == NULL)
	{
		format = "%s : %s\n";
	}

	fprintf(stderr, format, gettext(mesg), detail);
}

static void load_rc(int errmsg, char **dbname)
{
	FILE *fd;
	char *line, pom[2048];

	fd = open_rc("r");

	if (fd == NULL)
	{
		return;
	}

	while ((line = fgets(pom, sizeof(pom), fd)) != NULL)
	{
		static const char parse_msg[] = gettext_nop("Unable to parse");
		static const char parse_fmt[] = "%s : \"%s\"\n";
		static const char white_spaces[] = " \t";

		int i;

		line += strspn(line, white_spaces);

		if (*line == '#' || (i = strcspn(line, "\n")) == 0)
		{
			continue;
		}

		line[i]	= '\0';
		i	= NSC_LAST;

		do
		{
			int		len;
			const char	*str;

			str = cfgrc_name[i];
			len = strlen(str);

			if (g_strncasecmp(line, str, len) == 0)
			{
				line += len;
				line += strspn(line, white_spaces);
				break;
			}
		}
		while (--i >= NSC_FIRST);

		switch (i)
		{
			case NSC_TYPE:
			{
				int type = get_str_index(nscache_cache_type_str, line);

				if (type < 0)
				{
					if (errmsg)
					{
						fprn_stderr(NULL, illegal_cache_type_str, line);
					}

					break;
				}

				nscache_db.type = type;
				break;
			}

			case NSC_INDEXDB:
			{
				if (*line != '\0')
				{
					*dbname = g_strdup(line);
				}

				break;
			}

			case NSC_VIEWER:
			{
				apassign *ad;

				ad = g_malloc(sizeof(apassign));
				apassign_data = g_slist_append(apassign_data, ad);

				ad->mimetype  = get_apassign_str(line);
				ad->args_file = get_apassign_str(NULL);
				ad->args_url  = get_apassign_str(NULL);

				break;
			}

			case NSC_MENUACCEL:
			{
				if (gaccel_parse_str(line) == FALSE && errmsg)
				{
					fprn_stderr(parse_fmt, parse_msg, line);
				}

				break;
			}

			case NSC_PROP:
			{
				if (gprop_parse(line) == FALSE && errmsg)
				{
					fprn_stderr(parse_fmt, parse_msg, line);
				}

				break;
			}

			default:
				if (errmsg)
				{
					fprn_stderr(parse_fmt, parse_msg, line);
				}

				break;
		}
	}

	fclose(fd);
}

static void print_version_help(int what, const char *prgname)
{
	if (what & kPrintVersion)
	{
		printf("%s-%s\n", nscache_package_str, nscache_version_str);
	}

	if ( ! (what & kPrintHelp))
	{
		exit(0);
	}

	printf(gettext("Usage: %s [OPTION]\n"), prgname);

	printf(
#ifdef CONFIG_BERKELEY_DB
#ifdef HAVE_MSIE
		"  -t,  --db-type=[ns|ns6|moz|ie|unk]\n"
#else
		"  -t,  --db-type=[ns|ns6|moz|unk]\n"
#endif
#else /* ! CONFIG_BERKELEY_DB */
#ifdef HAVE_MSIE
		"  -t,  --db-type=[moz|ie|unk]\n"
#else
		"  -t,  --db-type=[moz|unk]\n"
#endif
#endif
		"                           %s\n"

#ifdef CONFIG_BERKELEY_DB
		"                           ns  = %s\n"
		"                           ns6 = %s\n"
		"                                 %s\n"
#endif
		"                           moz = %s\n"
		"                                 %s\n"
#ifdef HAVE_MSIE
		"                           ie  = %s\n"
#endif
		"                           unk = %s\n"
		"  -f,  --db-file=%s %s\n"
		"  -d,  --temp-dir=%s %s\n"
		"  -s,  --summary           %s\n"
		"  -l,  --list              %s\n"
		"  -m,  --multi-line        %s\n"
		"  -S,  --sort=[url|file|size|access|mime]\n"
		"                           %s\n"
		"  -R,  --sort-reverse      %s\n"
#ifndef CACHE_READONLY
		"  -r,  --readonly          %s\n"
#endif
		"  -v,  --version           %s\n"
		"  -h,  --help              %s\n",
		gettext("set cache type"),
#ifdef CONFIG_BERKELEY_DB
			gettext(NSC_GET_BROWSER_NAME(NSCACHE_RT_NETSCAPE_4)),
			gettext(NSC_GET_BROWSER_NAME(NSCACHE_RT_NETSCAPE_6)),
			gettext("(also Mozilla < 0.9)"),
#endif
			gettext(NSC_GET_BROWSER_NAME(NSCACHE_RT_MOZILLA)),
			gettext("(also Firefox, SeaMonkey and Netscape)"),
#ifdef HAVE_MSIE
			gettext(NSC_GET_BROWSER_NAME(NSCACHE_RT_MSIE)),
#endif
			gettext("unknown (default)"),
		gettext("FILENAME "),
			gettext("set cache file to FILENAME"),
		gettext("DIR     "),
			gettext("store temporary files in DIR (default: /tmp)"),
		gettext("print number of entries and total size"),
		gettext("list cache directory contents to stdout"),
		gettext("print multi line info about each cache entry"),
		gettext("print cache entries sorted by type (default: url)"),
		gettext("print cache entries in reverse order"),
#ifndef CACHE_READONLY
		gettext("activate readonly mode"),
#endif
		gettext("display version information and exit"),
		gettext("display this help and exit")

	); /* printf() */

	exit(0);
}

int cmd_setup(int argc, char **argv, char **dbname)
{
	const struct option options[] =
	{
		{"db-type", 1, NULL, 't'},
		{"db-file", 1, NULL, 'f'},
		{"temp-dir", 1, NULL, 'd'},
		{"summary", 0, NULL, 's'},
		{"list", 0, NULL, 'l'},
		{"multi-line", 0, NULL, 'm'},
		{"sort", 1, NULL, 'S'},
		{"sort-reverse", 0, NULL, 'R'},
#ifndef CACHE_READONLY
		{"readonly", 0, NULL, 'r'},
#endif
		{"help", 0, NULL, 'h'},
		{"version", 0, NULL, 'v'},
		/* !!! GTK&GDK OPTIONS !!! */
		{"gtk-module", 1, NULL, 'g'},
		{"g-fatal-warnings", 0, NULL, 'g'},
		{"gtk-debug", 2, NULL, 'g'},
		{"gtk-no-debug", 2, NULL, 'g'},
		{"gdk-debug", 2, NULL, 'g'},
		{"gdk-no-debug", 2, NULL, 'g'},
		{"display", 1, NULL, 'g'},
		{"sync", 0, NULL, 'g'},
		{"no-xshm", 0, NULL, 'g'},
		{"name", 1, NULL, 'g'},
		{"class", 1, NULL, 'g'},
		{"gxid_host", 1, NULL, 'g'},
		{"gxid_port", 1, NULL, 'g'},
		{"xim-preedit", 1, NULL, 'g'},
		{"xim-status", 1, NULL, 'g'},
		{NULL, 0, NULL, 0},
	};

	/* viewer.c */
	extern char *viewer_temp_dir;

	int	print_list, print_info, sort_reverse;
	char	*db_name;
	char	*db_type;
	char	*sort_type;
	char	*temp_dir;

	print_list = kNoStdOut;
	print_info = 0;
	sort_reverse = 0;

	db_name	   = NULL;
	db_type    = NULL;
	sort_type  = NULL;
	temp_dir   = NULL;

	*dbname		= NULL;
	nscache_db.type	= NSCACHE_RT_UNKNOWN;

	for(;;)
	{

#ifndef CACHE_READONLY
#define OPTION_RDONLY	"r"
#else
#define OPTION_RDONLY
#endif

		int c, index;

		c = getopt_long(argc, argv, OPTION_RDONLY "vhlsmRd:f:t:S:", options, &index);

		if (c == -1)
		{
			break;
		}

		switch (c)
		{
			case 'v':
				print_info |= kPrintVersion;
				break;
			case 'h':
				print_info |= kPrintHelp;
				break;
			case 'f':
				db_name = optarg;
				break;
			case 't':
				db_type = optarg;
				break;
			case 'd':
				temp_dir = optarg;
				break;
			case 'm':
				print_list = kOnlyStdOutMultiLine;
				break;
			case 'l':
				print_list = kOnlyStdOutList;
				break;
			case 's':
				print_list = kOnlyStdOutSummary;
				break;
			case 'S':
				sort_type = optarg;
				break;
			case 'R':
				sort_reverse = STYPE_REVERS_MASK;
				break;

#ifndef CACHE_READONLY
			case 'r':
				nscache_db.readonly = TRUE;
				break;
#endif
			case 'g':
				break;

			default:
				return EINVAL;
		}
	}

	if (optind < argc)
	{
		fprn_stderr(NULL, gettext_nop("Illegal argument"), argv[optind]);
		return EINVAL;
	}

	if (print_info)
	{
		print_version_help(print_info, argv[0]);
	}

	load_rc(print_list == kNoStdOut, dbname);

	if (db_name)
	{
		g_free(*dbname);

		*dbname		= g_strdup(db_name);
		nscache_db.type	= NSCACHE_RT_UNKNOWN;
	}

	if (db_type)
	{
		int type = get_str_index(nscache_cache_type_str, db_type);

		if (type < 0)
		{
			fprn_stderr(NULL, illegal_cache_type_str, db_type);
			return EINVAL;
		}

		nscache_db.type = type;
	}

	if (print_list)
	{
		int error, stype;

		if (*dbname == NULL
#ifdef HAVE_MSIE
			&& (nscache_db.type != NSCACHE_RT_MSIE)
#endif
			)
		{
			fprn_stderr("%s%s", gettext_nop("Cache file not specified!"), "\n");
			return EINVAL;
		}

		stype = STYPE_URL;

		if (sort_type)
		{
			stype = get_str_index(nscache_sort_type_str, sort_type);

			if (stype < 0)
			{
				fprn_stderr(NULL, gettext_nop("Illegal sort type"), sort_type);
				return EINVAL;
			}
		}

		error = nscache_dump(*dbname, print_list, stype | sort_reverse);
		exit(error);
	}


	if (temp_dir)
	{
		struct stat	tmpstat;
		const char	*home = NULL;

		if (temp_dir[0] == '~' &&
			(temp_dir[1] == '\0' || temp_dir[1] == '/'))
		{
			temp_dir++;
			home = g_get_home_dir();
		}

		if (home == NULL)
		{
			home = "";
		}

		temp_dir = g_strconcat(home, temp_dir, NULL);

		if ( stat(temp_dir, &tmpstat) ||
			! (S_ISDIR(tmpstat.st_mode)) ||
				access(temp_dir, X_OK | W_OK))
		{
			g_free(temp_dir);
			temp_dir = NULL;
		}
	}

	if (temp_dir == NULL)
	{
		temp_dir = strdup(g_get_tmp_dir());
	}

	if (temp_dir)
	{
		int len = strlen(temp_dir) - 1;

		while (len >= 0 && temp_dir[len] == '/')
		{
			temp_dir[len] = '\0';
			len--;
		}

		viewer_temp_dir = temp_dir;
	}

	if (*dbname == NULL)
	{
		const char *home = g_get_home_dir();

		if (home)
		{

#ifdef CONFIG_BERKELEY_DB
			*dbname = g_strconcat(home, "/.netscape/cache", NULL);

			if (access(*dbname, F_OK) == 0)
			{
				return 0;
			}

			g_free(*dbname);
#endif
			*dbname = g_strconcat(home, "/.mozilla", NULL);
		}
	}

	return 0;
}

/* EOF */
