/***************************************************************************/
/* 		This code is part of Nscache - viewer of Netscape(tm)	   */
/*		browsers disk cache					   */
/*		Copyright (c) 1999,2000 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		modified 2005,2006,2007 by Harald Foerster		   */
/*		(harald_foerster@users.sourceforge.net)			   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include "nscache.h"
#include "file.h"

#include <ctype.h>
#include <string.h>

#ifdef HAVE_GLOB
#include <glob.h>
#endif

#ifdef __powerpc__
#define PRE_INCREMENT
#endif

char* tl_igncase_strstr(const char *str, const char *pattern)
{
	/*
		Look for occurence of 'pattern' in 'str',
		letters in 'pattern' must be upper case.
	*/

	unsigned int firstc, c1;

#ifdef PRE_INCREMENT

	firstc = *pattern;

	if (firstc == '\0')
	{
		return (char *) str;
	}

	str--;

	while((c1 = *(++str)) != '\0')
#else

	firstc = *(pattern++);

	if (firstc == '\0')
	{
		return (char *) str;
	}

	while((c1 = *(str++)) != '\0')
#endif
	{
		if(toupper(c1) == firstc)
		{
			unsigned int c2;

			const unsigned char *s = str;
			const unsigned char *p = pattern;
			
			do
			{
#ifdef PRE_INCREMENT
				c1 = toupper(*(++s));
				c2 = *(++p);
#else
				c1 = toupper(*(s++));
				c2 = *(p++);
#endif
			}
			while(c1 == c2 && c1);
			
			if(c2 == '\0')
			{
#ifdef PRE_INCREMENT
				return (char*) str;
#else
				return (char*) str - 1;
#endif
			}

			str++;
			
			if(c1 == '\0')
			{
				break;
			}
		}

	} /* while((c1 = *(str++)) != '\0') */
	
	return NULL;
}

char *tl_mem_find_str(const char *mem, int mlen, const char *str, int slen)
{
	const char *end = &mem[mlen - slen];

	/* while mlen > slen + terminating '\0' */

	while ((mlen = end - mem) > 0)
	{
		char *s = memchr(mem, *str, mlen);

		if (s == NULL)
		{
			break;
		}

		mem = s;

		/* if mlen > slen + terminating '\0' */

		if (end > mem)
		{
			/* without first char, but with terminating '\0' */

			if (memcmp(&mem[1], &str[1], slen) == 0)
			{
				return (char *) mem;
			}

			mem++;
		}
	}

	return NULL;
}

char *tl_time_to_str(char* buf, size_t len, const time_t *ptime)
{
	if (*ptime)
	{
		struct tm *ptm;

		ptm = localtime(ptime); 
		strftime(buf, len, "%H:%M %d.%m.%Y", ptm);

		return buf;
	}

	*buf = '\0';

	return NULL;
}

char *tl_get_url_start(const char *urlstr)
{
	/*
		If we've got something like this
		"id=123456b&uri=http://www.foo.bar/index.php"
		then we'll skip the first part of the URL and use
		"http://www.foo.bar/index.php"
	*/

	char *beg = strstr(urlstr, "://");

	if (beg)
	{
		while (beg > urlstr && (isalnum(beg[-1]) || beg[-1] == '-'))
		{
			beg--;
		}
	}

	return beg;
}

int tl_str_escape_chars(char *buf, int len, const char *str, const char *esc_chars)
{
	int	chr;
	char	*beg, *end;

	if (len <= 0)
	{
		return -1;
	}

	if (esc_chars == NULL)
	{
		esc_chars = "\0";
	}

	chr = '\0';
	beg = buf;

	if (str)
	{
		end = &buf[len - 1];

		while ((chr = *(str++)) != '\0' && buf < end)
		{
			if (strchr(esc_chars, chr))
			{
				*(buf++) = '\\';

				if (buf >= end)
				{
					break;
				}
			}

			*(buf++) = chr;
		}
	}

	*buf = '\0';

	return (chr == '\0') ? buf - beg : -1;
}

char *tl_get_1qstr(char *str)
{
	static char *ptr = NULL;

	if (str == NULL)
	{
		str = ptr;
	}

	if (str)
	{
		unsigned int chr;

		char *beg = NULL;

		while ( (chr = *(str++)) != '\0' )
		{
			if (chr =='\\')
			{
				memmove(&str[-1], str, strlen(str) + 1);
			}

			else if (chr =='\"')
			{
				if (beg)
				{
					str[-1] = '\0';
					ptr = str;
					return beg;
				}

				beg = str;
			}
		}
	}

	ptr = NULL;

	return NULL;
}

#ifdef HAVE_GLOB

CompletitonData *tl_fname_completion(const gchar *fname)
{
	static const char esc_chars[] = "\\*?[]";

	int		len;
	size_t		idx, path_len, pattern_len;
	char		*beg, *str;
	CompletitonData	*retv;
	char		buf[4096];

	if (fname == NULL)
	{
		return NULL;
	}

	str = strrchr(fname, '/');

	pattern_len = strlen(str == NULL ? fname : &str[1]);

	idx = 0;
	path_len = 0;

	if (*fname == '~' && fname[1] == '/')
	{
		const char *home = g_get_home_dir();

		if (home)
		{
			len = tl_str_escape_chars(buf,
					sizeof(buf) - 1, home, esc_chars);

			if (len < 0)
			{
				return NULL;
			}

			idx = 2;
			buf[len] = '/';
			path_len = len + 1;
		}
	}

	len = tl_str_escape_chars(&buf[path_len],
			sizeof(buf) - 1 - path_len, &fname[idx], esc_chars);
	if (len < 0)
	{
		return NULL;
	}

	path_len += len;
	buf[path_len] = '*';
	buf[path_len + 1] = '\0';

	retv = g_malloc(sizeof(CompletitonData));
	retv->globdata = g_malloc(sizeof(glob_t));

	retv->fnamev = NULL;
	retv->append = NULL;

	if (glob(buf, (GLOB_ERR | GLOB_MARK), NULL, retv->globdata) ||
				((glob_t *) retv->globdata)->gl_pathc == 0)
	{
		tl_completion_free(retv);
		return NULL;
	}

	/* copy pointers to filenames, skip the path */

	beg = ((glob_t *) retv->globdata)->gl_pathv[0];
	str = strrchr(beg, '/');

	if (str == NULL)
	{
		/* filename */
		path_len = 0;
	}

	else
	{
		if (str[1] == '\0')
		{
			/* directory */
			while (*(--str) != '/')
			{
				if (str <= beg)
				{
					break;
				}
			}
		}

		if (*(str) == '/')
		{
			/* skip slash */
			str++;
		}

		path_len = str - beg;
	}

	retv->fnamec = ((glob_t *) retv->globdata)->gl_pathc;
	retv->fnamev = g_malloc((retv->fnamec + 1) * sizeof(char*));

	idx = 0;

	do
	{
		str = ((glob_t *) retv->globdata)->gl_pathv[idx];
		retv->fnamev[idx] = &str[path_len];

	}
	while(++idx < retv->fnamec);

	retv->fnamev[idx] = NULL;

	/*
		if single match, 'append' points
		to start of new characters
	*/

	retv->append = retv->fnamev[0] + pattern_len;

	if (retv->fnamec > 1)
	{
		/*
			else 'append' points to a copy of the
			first matching characters from start
		*/

		int pos;

		pos = pattern_len;
		beg = retv->fnamev[0];

		for (;;)
		{
			int	chr;
			char	**array;

			chr   = beg[pos];
			array = &retv->fnamev[1];

			while ((str = *(array++)) != NULL)
			{
				if (str[pos] != chr)
				{
					goto __done;
				}
			}

			pos++;
		}
__done:
		retv->append = g_strndup(retv->append, pos - pattern_len);
	}

	return retv;
}

void tl_completion_free(CompletitonData *ptr)
{
	if (ptr)
	{
		g_free(ptr->fnamev);

		if (((glob_t *) ptr->globdata)->gl_pathc > 1)
		{
			g_free(ptr->append);
		}

		globfree(ptr->globdata);
		g_free(ptr->globdata);

		g_free(ptr);
	}
}
#endif /* HAVE_GLOB */

#ifdef CONFIG_BERKELEY_DB
char *tl_strnrsep(char *str, int len, int *sep)
{
	int c = *sep;

	if (c && len > 0)
	{
		int  n;
		char *s;

		if (c < 0)
		{
			static const char sep_str[] = "/\\:";

			const char *p = sep_str;

			while (s = &str[len], (c = *(p++)) != '\0')
			{
				while (s > str)
				{
					if (*(--s) == c)
					{
						break;
					}
				}

				if (s > str)
				{
					break;
				}
			}

			*sep = c;
		}

		n = 2;
		s = &str[len];

		while (s > str)
		{
			if (*(--s) == c)
			{
				*s = '/';

				if (--n <= 0)
				{
					return &s[1];
				}
			}
		}

	} /* if (c && len > 0) */

	return str;
}
#endif /* CONFIG_BERKELEY_DB */

/* EOF */
