/***************************************************************************/
/* 		This code is part of Nscache - viewer of Netscape(tm)	   */
/*		browsers disk cache					   */
/*		Copyright (c) 1999,2000 Ondrejicka Stefan		   */
/*		(ondrej@idata.sk)					   */
/*		created and modified 2005,2006,2007 by Harald Foerster	   */
/*		(harald_foerster@users.sourceforge.net)			   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "file.h"
#include "stringbuf.h"
#include "viewer.h"


/* temporary file name */
const char *viewer_temp_dir = "/tmp";

/* parameter is URL or file */
const char uof_param_str[][3] = {"%u", "%f"};


/* error types */

enum
{
	FORK_ERROR = -1,
	EXEC_ERROR =  1
};


/* wait until child has sent SIGUSR1 */

enum
{
	SIGUSR1_WAIT = -1,
	SIGUSR1_DONE =  0
};


/* fetch SIGUSR1 and set status to DONE */

volatile int	viewer_status;
volatile pid_t	viewer_pid;


/* allocated for each started viewer examining a temporary file */

typedef struct
{
	pid_t	child;
	char*	temp_file;

} viewer_data;


static pid_t parent_pid = 0;
static pid_t parent_uid = 0;

static GSList* children	= NULL;




static void unlink_temporary_file(gpointer slist_data, gpointer user_data)
{
	/* Remove temporary file */

	viewer_data* viewer_ptr	= slist_data;

	unlink(viewer_ptr->temp_file);

} /* static void unlink_temporary_file(gpointer, gpointer) */



static void viewer_atexit(void)
{
	/* Remove all temporary files */

	g_slist_foreach(children, unlink_temporary_file, NULL);

} /* static void viewer_atexit(void) */



static void viewer_sigterm(int signum)
{
	/* This is the end => but call 'viewer_atexit()' before it */

	viewer_atexit();
	exit(0);

} /* static void viewer_sigterm(int signum) */



static int search_child(gconstpointer slist_data, gconstpointer user_data)
{
	/* Compare viewer pid with user_data and return zero if equal */

	const viewer_data* viewer_ptr = slist_data;

	pid_t pid = *(pid_t *) user_data;

	return viewer_ptr->child - pid;

} /* static int search_child(gconstpointer, gconstpointer) */



static void viewer_sigchld(int signum)
{
	/* Remove temporary file and free memory  */

	int	status;
	pid_t	pid;

	while( (pid = waitpid(-1, &status, WNOHANG)) > 0 )
	{
		GSList*	slist;

		if(viewer_pid == pid)
		{
			/*
				The viewer couldn't either be started
				or finished immediatly with an error.
			*/

			viewer_status	= WEXITSTATUS(status);
			viewer_pid	= SIGUSR1_DONE;
		}


		slist = g_slist_find_custom(children, &pid, search_child);

		if(slist)
		{
			viewer_data* viewer_ptr = slist->data;

			unlink(viewer_ptr->temp_file);
			free(viewer_ptr->temp_file);
			free(viewer_ptr);
			children = g_slist_remove(children, viewer_ptr);

		} /* if(slist) */

	} /* while((pid = waitpid(-1, &status, WNOHANG) > 0) */

} /* static void viewer_sigchld(int signum) */



static void viewer_sigusr1(int signum)
{
	/*
		Child is running and signaled us 'SIGUSR1'
		Maybe the viewer couldn't either be started
		or finished immediatly with an error.
		We will check this by calling 'viewer_sigchld',
		because 'SIGCHLD' is blocked at this moment.
	*/

	if(viewer_pid != SIGUSR1_DONE)
	{
		viewer_sigchld(SIGCHLD);
	}

	if(viewer_pid != SIGUSR1_DONE)
	{
		viewer_status = SIGUSR1_DONE;
	}

} /* static void viewer_sigusr1(int signum) */



static void viewer_signal_init(void)
{
	struct sigaction sigact;

	parent_pid = getpid();
	parent_uid = getuid();

	/* block other signals, while executing 'sa_handler' */
	sigfillset(&sigact.sa_mask);
	sigact.sa_flags = 0;

	/* MozCache has terminated */
	atexit(viewer_atexit);
	sigact.sa_handler = viewer_sigterm;
	sigaction(SIGTERM, &sigact, NULL);

	/* child was started */
	sigact.sa_handler = viewer_sigusr1;
	sigaction(SIGUSR1, &sigact, NULL);

	/* child was terminated */
	sigact.sa_handler = viewer_sigchld;
	sigact.sa_flags = SA_NOCLDSTOP;
	sigaction(SIGCHLD, &sigact, NULL);

} /* static void viewer_signal_init(void) */



static int start_viewer(const char *application, char *url_or_file,
						int uof_len, int viewer_type)
{
	pid_t		child_pid;
	sigset_t	sig_block, sig_unblock;
	int		argc, error;
	unsigned int	chr, end, exec_len;
	char*		str;
	char*		uof_arg;
	const char*	uof_repl;
	char*		exec_args;
	char*		argv[100];

	exec_len 	= strlen(application) + 1;
	uof_repl	= uof_param_str[viewer_type];
	str		= (char *) application;

	while ((str = strstr(++str, uof_repl)) != NULL)
	{
		exec_len += uof_len;
	}

	exec_args = malloc(exec_len);

	if(exec_args == NULL)
	{
		return -1;
	}

	/* break 'application' in pieces */

	uof_arg = NULL;

	argc = 0;
	str  = exec_args;
	end  = ' ';
	chr  = *(application++);

	while(chr != '\0')
	{
		argv[argc] = str;
		argc++;

		if(argc >= (sizeof(argv) / sizeof(char*)) - 1)
		{
			/* this shouldn't happen */
			exit(ENOMEM);
		}

		do
		{
			if(chr == end)
			{
				break;
			}

			if(chr == '\\')
			{
				chr = *(application++);

				if(chr == '\0')
				{
					break;
				}
			}

			else if(chr == '%' && argc > 1)
			{
				if(*application == uof_repl[1] &&
						! isalnum(application[1]))
				{
					str = strcpy(str, url_or_file) + uof_len;
					uof_arg = url_or_file;
					application++;
					continue;
				}
			}

			*(str++) = chr;
		}
		while( (chr = *(application++)) != '\0');

		*(str++) = '\0';
		end	 = ' ';

		while(chr == end) 
		{
			chr = *(application++);
		}

		if(chr == '\'')
		{
			end = chr;
			chr = *(application++);
		}
	}

	if(uof_arg == NULL)
	{
		argv[argc] = url_or_file;
		argc++;
	}

	argv[argc] = NULL;


	/* block SIGCHLD and SIGUSR1 (in fact all signals) */

	sigfillset(&sig_block);
	sigprocmask(SIG_BLOCK, &sig_block, &sig_unblock);

	switch(child_pid = fork())
	{
		case 0:
			kill(parent_pid, SIGUSR1);
			execvp(argv[0], argv);
			_exit(errno == 0 ? ENOENT : errno);

		case -1:
			break;

		default:
			viewer_pid	= child_pid;
			viewer_status	= SIGUSR1_WAIT;

			if(viewer_type == VIEWER_FILE)
			{
				viewer_data* viewer_ptr;

				viewer_ptr = malloc(sizeof(viewer_data));

				if(viewer_ptr == NULL)
				{
					break;
				}

				viewer_ptr->child     = child_pid;
				viewer_ptr->temp_file = url_or_file;

				children = g_slist_append(children, viewer_ptr);
			}


			/* wait for child */

			while(viewer_status == SIGUSR1_WAIT)
			{
				sigsuspend(&sig_unblock);
			}


			free(exec_args);

			/* unblock SIGCHLD and SIGUSR1 */
			sigprocmask(SIG_SETMASK, &sig_unblock, NULL);

			errno = viewer_status;

			return errno ? EXEC_ERROR : 0;

	} /* switch( fork() )) */


	error = errno;

	free(exec_args);

	/* unblock SIGCHLD and SIGUSR1 */
	sigprocmask(SIG_SETMASK, &sig_unblock, NULL);

	errno = error;

	viewer_pid = SIGUSR1_DONE;

	return FORK_ERROR;

} /* static int start_viewer(const char *application, char *url_or_file,
							int uof_len, int viewer_type) */



int viewer_start(const char* appassign, const nscache_record_t* rec, int viewer_type)
{
	static const char time_fmt_str[] = "%d%m%H%M%S";

	int		error, length;
	time_t		tm;
	char		buf[256];
	StringBuf	temp;

	if(appassign == NULL || rec == NULL)
	{
		errno = EINVAL;
		return -1;
	}

	if(parent_pid == 0)
	{
		viewer_signal_init();
	}

	if(viewer_type == VIEWER_URL)
	{
		return start_viewer(appassign, rec->urlstr,
					strlen(rec->urlstr), viewer_type);
	}

	/* uid, pid and length*/
	length = sprintf(buf, "%X%X%X",
			parent_uid, parent_pid, rec->content_length);

	/* append last access */
	length  += strftime(&buf[length], sizeof(buf) - length,
			time_fmt_str, localtime(&(rec->last_accessed)));

	/* append date and time */
	tm	 = time(NULL);
	length  += strftime(&buf[length], sizeof(buf) - length,
					time_fmt_str, localtime(&tm));

	/* now try to create an unique file name */
	stringbuf_init(&temp, viewer_temp_dir, 0);
	stringbuf_append(&temp, "/mozcache-", sizeof("/mozcache-") - 1);
	stringbuf_append(&temp, buf, length);

	error = file_copy(rec, temp.str, FILE_PERM_TEMP);

	if (error)
	{
		stringbuf_delete(&temp);
	}

	else
	{
		error = start_viewer(appassign, temp.str, temp.len, VIEWER_FILE);

		if(error == FORK_ERROR && temp.str)
		{
			unlink(temp.str);
			stringbuf_delete(&temp);
		}
	}

	return error;

} /* int viewer_start(const char* appassign, const nscache_record_t* rec, int viewer_type) */

/* EOF */
